#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
    A code for ploting figure by reading xpm file generated by gromacs tools, such as gmx mdmat, gmx sham,
    gmx rms, gmx do_dssp, gmx covar etc.

    @update date: 2021.08.18
    @version:     1.5
"""

import argparse
import sys

import matplotlib.colors as colors
import matplotlib.pyplot as plt
import numpy as np


def Read_Plot_XPM(args):
    color, x, y, arr, value_list, ss, color_map = {}, [], [], [], [], [], []
    title, legend, xlabel, ylabel = '', '', '', ''
    ncode = 1                                       # default letter length for representing color
    _ss = ['Coil', 'B-Sheet', 'B-Bridge', 'Bend', 'Turn',
           'A-Helix', '5-Helix', '3-Helix', 'Chain_Separator']
    _color_map = ['#FFFFFF', '#FF0000', '#000000', '#008000', '#FFFF00',
                  '#0000FF', '#800080', '#808080', '#E6E6E6']
    with open(args.file, 'r') as f:
        line = f.readline().rstrip('\n')
        while line:
            if 'title' in line:
                title = line.split('"')[1]
            elif 'legend' in line:
                legend = line.split('"')[1]
            elif 'x-label' in line:
                xlabel = line.split('"')[1]
            elif 'y-label' in line:
                ylabel = line.split('"')[1]
            # 矩阵维度dim[0]*dim[1], nframe * nres
            elif 'static char' in line:
                dim = np.array(f.readline().split('"')[1].split(), dtype=int)
                if dim[3] > 1:
                    ncode = dim[3]
                for i in range(dim[2]):
                    temp = f.readline().rstrip('\n')
                    value = temp.split('"')[-2]
                    value_list.append(value)
                    if 'Secondary' not in title:
                        color[temp[1:1+ncode]] = float(value) if 'Hydrogen' not in ylabel else 0 if value == 'None' else 1
                    else:
                        color[temp[1:1+ncode]] = i
                for j in _ss:
                    if j in value_list:
                        ss.append(j)
                        color_map.append(_color_map[_ss.index(j)])
            # 坐标值
            elif 'x-axis' in line:
                x.extend(line.split()[2:-1])
            elif 'y-axis' in line:
                y.extend(line.split()[2:-1])
            # 符号矩阵转实际值
            elif line[0] == '"' and line[dim[0]*ncode + 1] == '"':
                temp = [color[line[j:j+ncode]] for j in range(1, dim[0]*ncode + 1, ncode)]
                arr.append(temp)
            line = f.readline().rstrip('\n')
        # 必须按行反向列表，因为xpm中的数据默认是反向的，即最后一行数据才是开始值
        '''
        Ref:
            http://manual.gromacs.org/documentation/current/reference-manual/file-formats.html#xpm
            https://github.com/Becksteinlab/GromacsWrapper/issues/125
        ^
        res N
        |
        |
        |
        res 1
        0----------------time--------->
        '''
        arr.reverse()
    # 处理一下维度，因为有的xpm文件坐标个数会大于下面的矩阵维度dim[0](如gmx sham)
    if len(x) > dim[0]:
        print('Process coordinate axis dimension to match array...')
        x.pop()
        y.pop()

    # 绘图
    plt.figure(figsize=(8, 6))
    x = np.array(x, dtype=float)
    y = np.array(y, dtype=float)
    X, Y = np.meshgrid(x, y)
    Z = np.array(arr, dtype=float)
    # 氢键map图
    if "Hydrogen" in ylabel:
        h = plt.contourf(X, Y, Z, np.linspace(0, 1, 3), colors=['white', 'red'])
        cb = plt.colorbar(h, orientation='horizontal', fraction=0.03)
        cb.set_ticks([0.25, 0.75])
        cb.set_ticklabels(['None', 'Present'])
    # 蛋白二级结构图
    elif 'Secondary' in title:
        # fix plot bug, we should use pcolormesh
        cmap = colors.ListedColormap(color_map)
        boundaries = list(color.values())
        boundaries.append(len(boundaries))
        norm = colors.BoundaryNorm(boundaries, cmap.N)
        h = plt.pcolormesh(X, Y, Z, cmap=cmap, norm=norm, shading='nearest')
        cb = plt.colorbar(h, orientation='horizontal')
        cb.set_ticks(np.array(boundaries[:-1]) + 0.5)
        cb.set_ticklabels(value_list)
    # 其他类型的图
    else:
        h = plt.contourf(X, Y, Z, np.linspace(min(color.values()), max(color.values()), 9), cmap="jet")
        cb = plt.colorbar(h)
    cb.ax.tick_params(labelsize=13)
    cb.set_label(legend, fontsize=18)

    # 设置显示
    plt.title(title, fontsize=18)
    plt.xlabel(xlabel, fontsize=18)
    plt.ylabel(ylabel, fontsize=18)
    plt.xticks(fontsize=15)
    plt.yticks(fontsize=15)

    # 根据命令行设置显示范围
    if not args.xrange:
        plt.xlim(np.min(x), np.max(x))
    else:
        xmin = float(args.xrange.split()[0])
        xmax = float(args.xrange.split()[1])
        plt.xlim(xmin, xmax)
    if not args.yrange:
        plt.ylim(np.min(y), np.max(y))
    else:
        ymin = float(args.yrange.split()[0])
        ymax = float(args.yrange.split()[1])
        plt.ylim(ymin, ymax)

    plt.minorticks_on()

    if not args.save:
        plt.show()
    else:
        plt.savefig(args.file.split('.')[0] + '.png', dpi=600)


def ParseOptions():
    parser = argparse.ArgumentParser(description='This is a script for reading gromacs xpm file and plot figure')
    parser.add_argument('-f', '--file', action="store", default='ss.xpm', metavar='xxx.xpm',
                        help='Input xpm file generated by gromacs tools, such as gmx mdmat, \
                                gmx sham, gmx rms, gmx do_dssp, gmx covar etc (default: ss.xpm)')
    parser.add_argument('-s', '--save', action='store_true',
                        help='If given, save figure to png file with 600 dpi (default: False)')
    parser.add_argument('-x', '--xrange',
                        help='The range for x axis (default: min(x) max(x))')
    parser.add_argument('-y', '--yrange',
                        help='The range for y axis (default: min(y) max(y))')
    args = parser.parse_args()
    if len(sys.argv) < 2:
        parser.print_help()
        print('\n  For example: \n\tpython %s -f ss.xpm -x "0 1000" -y "1 50"' % (sys.argv[0]))
        sys.exit(1)

    return args


if __name__ == '__main__':
    Read_Plot_XPM(ParseOptions())


